/* KoreMotor Library for Koala
 */ 
  
  
#ifndef __LIBKMOT
#define __LIBKMOT

/* Regulation types */
typedef enum {reg_open=0,reg_pos=1,reg_posprofile=2,reg_speed=3,reg_current=5} RegType;
/* Measurements */
typedef enum {mes_pos=1, mes_speed=3, mes_current=5} Measure;
/* Target point sources */
typedef enum {src_ext=0, src_sqr=1, src_tri=2, src_sin=3} SetPointSource;

/* Options flags definitions*/
#define SW_SEPARATED    1
#define SW_WINDUP       2
#define SW_STOPMIN      4
#define SW_STOPMAX      8
#define SW_STOP_ERROR   16
#define SW_BLOCKED_STOP 32
#define SW_CURRENT_CTRL 64
#define SW_DIR_INV      128

#define HW_STARTUP    1
#define HW_ANALOG_SET 2
#define HW_LED        4
#define HW_RESOLUTION 8
#define HW_TORQUE_INV 16
#define HW_OPT1       32
#define HW_OPT2       64
#define HW_OPT3       128


/* Functions */
int kmot_I2cWrite(int8 mot, int8 addr, int8 data);
int kmot_I2cRead(int8 mot, int8 addr);
int kmot_I2cRead32(int8 mot, int8 addr, int32* value);
int kmot_I2cRead16(int8 mot, int8 addr, int16* value);

int kmot_TestLink();
int kmot_ResetError(int8 mot);
int kmot_SetPoint(int8 mot, RegType reg, int32 setpoint);
int kmot_GetMeasure(int8 mot, Measure mes, int32* value);
int kmot_SetPositionLimit(int8 mot, int32 min, int32 max);
int kmot_SetCurrentLimit(int8 mot, int32 max);
int kmot_ConfigurePID(int8 mot, RegType reg, int16 Kp, int16 Kd, int16 Ki);
int kmot_SaveConfig(int8 mot);
int kmot_SetPosition(int8 mot, int32 position);
int kmot_SetSpeedProfile(int8 mot, int16 maxspeed, int8 acceleration);
int kmot_SearchLimits(int8 mot,int8 blocked_time,int32 speed,int32* minpos,int32* maxpos);
int kmot_SetPointSource(int8 mot, RegType reg, SetPointSource src,int8 period, int8 amplitude, int8 offset);

int kmot_GetStatus(int8 mot, int8* statusflag, int8* errorflag);
int kmot_SetOptions(int8 mot, int8 hwoption, int8 swoption);
int kmot_SetBlockedTime(int8 mot,int8 time);
int kmot_SetSampleTime(int8 mot,int8 sample);
int kmot_SetFilterOrder(int8 mot,int8 order);
int kmot_ResetController(int8 mot);

#endif
