#include <sys/kos.h>
#include <stdio.h>
#include "libkoremotor.h"
//#include "libkoremotor.c"

/******************************************************************/
int32 minpos0,minpos1,maxpos0,maxpos1;
int32 minpos2,minpos3,maxpos2,maxpos3;
static int32 vIDProcess[6];

int started = 0;

#define PERIOD 2000
#define SPEED 10 
#define TIME  10

/******************************************************************/
void InitMotor(int8 mot)
{

    kmot_ResetController(mot);
    kmot_I2cWrite(mot,0x28,0);
    kmot_SetFilterOrder(mot,0);
#if 1
    kmot_SetSampleTime(mot,6);
    kmot_I2cWrite(mot,0x60,10);
    kmot_SetOptions(mot,0,SW_WINDUP|SW_BLOCKED_STOP|SW_STOP_ERROR|SW_STOPMAX|SW_STOPMIN);     
#else
    kmot_SetOptions(mot,HW_RESOLUTION,SW_WINDUP|SW_BLOCKED_STOP|SW_STOP_ERROR|SW_STOPMAX|SW_STOPMIN);
    kmot_SetSampleTime(mot,12);
    kmot_I2cWrite(mot,0x60,1);
#endif
    kmot_ResetError(mot);
    kmot_SetBlockedTime(mot,TIME);
    kmot_SetCurrentLimit(mot,50);
    kmot_SetPositionLimit(mot,-10000,10000);
}
void PrintMotorStatus(int8 mot)
{
    int32 speed,torque,position,setpoint;
    int8  errorflag, statusflag;

    kmot_GetStatus(mot,&statusflag,&errorflag);
    kmot_GetMeasure(mot,mes_speed,&speed);
    kmot_GetMeasure(mot,mes_pos,&position);
    kmot_GetMeasure(mot,mes_current,&torque);
    kmot_I2cRead32(mot,0x2F,&setpoint);
    printf("%d) error: %x status:%x speed: %ld pos: %ld current: %ld setpoint %ld\r\n",
	   mot,errorflag,statusflag,speed,position,torque,setpoint);
}

void print_status(int mot,char * str)
{
    if(started & mot)
	sprintf(str,"running");
    else
	sprintf(str,"idle   ");
}

void print_error(int8 err, char * str)
{
    if(err & 1)
    {
	sprintf(str,"Sample time error");
	return;
    }
    if(err & 2)
    {
	sprintf(str,"Watchdog error");
	return;
    }
    if(err & 4)
    {
	sprintf(str,"Brown-out error");
	return;
    }
    if(err & 8)
    {
	sprintf(str,"Emergency Stop");
	return;
    }
    if(err & 16)
    {
	sprintf(str,"Motor Blocked");
	return;
    }
    if(err & 32)
    {
	sprintf(str,"Pos out of range");
	return;
    }
    if(err & 64)
    {
	sprintf(str,"Speed out of range");
	return;
    }
    if(err & 128)
    {
	sprintf(str,"Torque out of range");
	return;
    }

    sprintf(str," ");
}

static void process_1 ()
{
    int8  errorflag0,errorflag1,errorflag2,errorflag3,statusflag;
    int8  old_error0 =0;
    int8  old_error1 =0;
    int8  old_error2 =0;
    int8  old_error3 =0;
    int   old_started=0;
    char  status0[8];
    char  status1[8];
    char  status2[8];
    char  status3[8];
    char  error0[20];
    char  error1[20];
    char  error2[20];
    char  error3[20];
    
    printf("\f");
    printf("Press q,w,e,r to start/stop motor 0,1,2,3\r\n");
    printf("Press z to reset errors for all motors\r\n");
    printf("0) idle\r\n");
    printf("1) idle\r\n");
    printf("2) idle\r\n");
    printf("3) idle\r\n");

    for(;;)
    {
	kmot_GetStatus(0,&statusflag,&errorflag0);
	kmot_GetStatus(1,&statusflag,&errorflag1);
	kmot_GetStatus(2,&statusflag,&errorflag2);
	kmot_GetStatus(3,&statusflag,&errorflag3);

	if(old_started != started    ||
           old_error0  != errorflag0 ||
           old_error1  != errorflag1 ||
           old_error2  != errorflag2 ||
           old_error3  != errorflag3 )
	{
	    print_status(1,status0);
	    print_status(2,status1);
	    print_status(4,status2);
	    print_status(8,status3);

	    print_error(errorflag0,error0);
	    print_error(errorflag1,error1);
	    print_error(errorflag2,error2);
	    print_error(errorflag3,error3);
	    
	    printf("\f");
	    printf("Press q,w,e,r to start/stop motor 0,1,2,3\r\n");
	    printf("Press z to reset errors for all motors\r\n");
	    printf("0) %s %s\r\n",status0, error0);
	    printf("1) %s %s\r\n",status1, error1);
	    printf("2) %s %s\r\n",status2, error2);
	    printf("3) %s %s\r\n",status3, error3);

	    old_started = started;
	    old_error0  = errorflag0;
	    old_error1  = errorflag1;
	    old_error2  = errorflag2;
	    old_error3  = errorflag3;
	}
	tim_suspend_task(500);
    }
}



/***********************************************************************/
/* Demo process for motor 2
 */ 
static void process_5 ()
{
    int8  mot = 2;
    
    kmot_SearchLimits(mot,2,3,&minpos2,&maxpos2);
    kmot_SetBlockedTime(mot,TIME);

    for(;;)
    {
	//kmot_SetPoint(mot,reg_posprofile,maxpos2-50);
	kmot_SetPoint(mot,reg_posprofile,maxpos2-200);
	tim_suspend_task(PERIOD);
	
	//kmot_SetPoint(mot,reg_posprofile,minpos2+50);
	kmot_SetPoint(mot,reg_posprofile,minpos2+200);
	tim_suspend_task(PERIOD);
	kmot_SetPositionLimit(mot,minpos2+10,maxpos2-10);
   }
}
/***********************************************************************/
/* Demo process for motor 3
 */ 
static void process_4 ()
{
    int8  mot = 3;
    int32 position;
    
    kmot_SearchLimits(mot,2,3,&minpos3,&maxpos3);
    kmot_SetBlockedTime(mot,TIME);
    
    for(;;)
    {
	kmot_SetPoint(mot,reg_posprofile,maxpos3-50);
	//kmot_SetPoint(mot,reg_posprofile,maxpos3-200);
	tim_suspend_task(PERIOD);
	kmot_GetMeasure(3,mes_pos,&position);
	
	kmot_SetPoint(mot,reg_posprofile,minpos3+50);
	//kmot_SetPoint(mot,reg_posprofile,minpos3+200);
	tim_suspend_task(PERIOD);
	kmot_SetPositionLimit(mot,minpos3+10,maxpos3-10);
    }
}

/***********************************************************************/
/* Demo process for motor 1
 */ 
static void process_3 ()
{
    int8  mot = 1;
    
    kmot_SearchLimits(mot,2,3,&minpos1,&maxpos1);
    kmot_SetBlockedTime(mot,TIME);
    
    for(;;)
    {
	//kmot_SetPoint(mot,reg_posprofile,maxpos1-50);
	kmot_SetPoint(mot,reg_posprofile,maxpos1-200);
	tim_suspend_task(PERIOD);

	//kmot_SetPoint(mot,reg_posprofile,minpos1+50);
	kmot_SetPoint(mot,reg_posprofile,minpos1+200);
	tim_suspend_task(PERIOD);
	kmot_SetPositionLimit(mot,minpos1+10,maxpos1-10);
    }
}

/***********************************************************************/
/* Demo process for motor 0
 */ 
static void process_2 ()
{
    int8  mot = 0;
    
    kmot_SearchLimits(mot,2,3,&minpos0,&maxpos0);
    kmot_SetBlockedTime(mot,TIME);
    
    for(;;)
    {
	//kmot_SetPoint(mot,reg_posprofile,maxpos0-50);
	kmot_SetPoint(mot,reg_posprofile,maxpos0-200);
	tim_suspend_task(PERIOD);

	kmot_SetPoint(mot,reg_posprofile,minpos0+200);
	//kmot_SetPoint(mot,reg_posprofile,minpos0+50);
	tim_suspend_task(PERIOD);
	kmot_SetPositionLimit(mot,minpos0+10,maxpos0-10);
    }
}
/***********************************************************************/
/* Serial commands for motors 
 */ 
static void process_0 ()
{
    char byte;
    int32 status;
    int i;

    for(;;)
    {
	byte = com_receive_byte();
	if(byte == 'z')
	{
	    for(i=0;i<4;i++)
	    {
		kmot_ResetError(i);
		kmot_ResetController(i);
	    }
	    printf("Error Reset\r\n");
	}
	if(byte == 'q')
	{
	    if(started & 0x0001)
	    {
		kill_task(vIDProcess[2]);

		started = started & 0xFFFE;

		printf("Motor 0 stopped\r\n");
	    }
	    else
	    {
		status = install_task("motor 0", 800, process_2);
		if (status == -1)
		{
		    printf("Error launching process\n");
		    exit (0);
		}

		vIDProcess[2] = (uint32)status;

		started = started | 0x0001;	
		
		printf("Motor 0 started\r\n");
	    }
	}
	if(byte == 'w')
	{
	    if(started & 0x0002)
	    {
		kill_task(vIDProcess[3]);

		started = started & 0xFFFD;

		printf("Motor 1 stopped\r\n");
	    }
	    else
	    {
		status = install_task("motor 1", 800, process_3);
		if (status == -1)
		{
		    printf("Error launching process\n");
		    exit (0);
		}

		vIDProcess[3] = (uint32)status;

		started = started | 0x0002;	
		
		printf("Motor 1 started\r\n");
	    }
	}
	
	if(byte == 'e')
	{
	    if(started & 0x0004)
	    {
		kill_task(vIDProcess[4]);

		started = started & 0xFFFB;

		printf("Motor 2 stopped\r\n");
	    }
	    else
	    {
		status = install_task("motor 2", 800, process_5);
		if (status == -1)
		{
		    printf("Error launching process\n");
		    exit (0);
		}

		vIDProcess[4] = (uint32)status;

		started = started | 0x0004;	
		
		printf("Motor 2 started\r\n");
	    }
	}
	if(byte == 'r')
	{
	    if(started & 0x0008)
	    {
		kill_task(vIDProcess[5]);

		started = started & 0xFFF7;

		printf("Motor 3 stopped\r\n");
	    }
	    else
	    {
		status = install_task("motor 3", 800, process_4);
		if (status == -1)
		{
		    printf("Error launching process\n");
		    exit (0);
		}

		vIDProcess[5] = (uint32)status;

		started = started | 0x0008;	
		
		printf("Motor 3 started\r\n");
	    }
	}
#if 0	
	if(byte == 'i')	
	{
	    printf("Kp:\r\n");
	    scanf("%ld",&Kp);
	    printf("Kd:\r\n");
	    scanf("%ld",&Kd);
	    printf("Ki:\r\n");
	    scanf("%ld",&Ki);

	    printf("Kp: %ld Kd: %ld Ki: %ld\r\n",Kp,Kd,Ki);
	    kmot_ConfigurePID(0,reg_speed,(int16)Kp,(int16)Kd,(int16)Ki);
	}	
	if(byte == 'j')	
	{
	    printf("Kp:\r\n");
	    scanf("%ld",&Kp);
	    printf("Kd:\r\n");
	    scanf("%ld",&Kd);
	    printf("Ki:\r\n");
	    scanf("%ld",&Ki);

	    printf("Kp: %ld Kd: %ld Ki: %ld\r\n",Kp,Kd,Ki);
	    kmot_ConfigurePID(1,reg_speed,(int16)Kp,(int16)Kd,(int16)Ki);
	}
#endif	
    }
}

/***********************************************************************/
int main (void)
{
    int32 status;

    tim_reset ();				// Reset multitasking manager
    mot_reset ();				// Reset Khepera movement manager
    ser_reset ();				// Reset serial link manager

    /* Init and setup for all motors */
    InitMotor(0);
    InitMotor(1);
    InitMotor(2);
    InitMotor(3);

    kmot_ConfigurePID(3,reg_speed,1500,0,300);
    kmot_ConfigurePID(3,reg_pos,70,50,10);
    kmot_SetSpeedProfile(3,SPEED,1);

    kmot_ConfigurePID(2,reg_speed,1500,0,300);
    kmot_ConfigurePID(2,reg_pos,70,50,10);
    kmot_SetSpeedProfile(2,SPEED,1);
    
    kmot_ConfigurePID(1,reg_speed,1500,0,300);
    kmot_ConfigurePID(1,reg_pos,70,50,10);
    kmot_SetSpeedProfile(1,SPEED,1);

    kmot_ConfigurePID(0,reg_speed,1650,0,320);
    kmot_ConfigurePID(0,reg_pos,70,50,10);
    kmot_SetSpeedProfile(0,SPEED,1);

    kmot_SaveConfig(0);
    kmot_SaveConfig(1);
    kmot_SaveConfig(2);
    kmot_SaveConfig(3);

    /* Launch all process */
    status = install_task ("serial control", 800, process_0);
    if (status == -1)
    {
        printf("Error launching process 0\n");
        exit (0);
    }
    
    vIDProcess[0] = (uint32) status;

    status = install_task ("print status", 800, process_1);
    if (status == -1)
    {
        printf("Error launching process 0\n");
        exit (0);
    }
    
    vIDProcess[1] = (uint32) status;

    return 0;
}
